import React from 'react';
import PropTypes from 'prop-types';
import { View, Text, TouchableOpacity, StyleSheet, Animated, FlatList, Easing } from 'react-native';
import { throttle } from 'lodash';
import { isNil } from 'ramda';
import { scaleH, screenWidth, scaleW } from '../utils/measure';
import { textStyle } from '../utils/styles';

const AnimatedFlatList = Animated.createAnimatedComponent(FlatList);

export default class HiddenFlatlist extends React.PureComponent {
  toggleHeight = 0;

  static propTypes = {
    renderItem: PropTypes.func.isRequired,
    titleStyle: View.propTypes.style,
  };

  static defaultProps = {
    defaultValue: true,
  };

  state = {
    rendered: false,
    toggle: this.props.defaultValue,
    toggleHeight: new Animated.Value(0),
    toggleMargin: this.props.defaultValue ? new Animated.Value(scaleH(20)) : new Animated.Value(0),
  };

  componentWillReceiveProps(nextProps) {
    if (!isNil(this.props.data)) {
      if (nextProps.data.length !== this.props.data.length) {
        this.setState({ rendered: false });
      }
    }
  }

  onListLayout = e => {
    if (!this.state.rendered) {
      this.toggleHeight = e.nativeEvent.layout.height;
      this.setState({
        toggleHeight: this.state.toggle
          ? new Animated.Value(this.toggleHeight)
          : new Animated.Value(0),
        rendered: true,
      });
    }
  };

  toggle = throttle(
    () => {
      Animated.parallel([
        Animated.timing(this.state.toggleHeight, {
          toValue: this.state.toggle ? 0 : this.toggleHeight,
          duration: this.props.duration,
          easing: this.props.easing && Easing.linear,
        }),
        Animated.timing(this.state.toggleMargin, {
          toValue: this.state.toggle ? 0 : scaleH(20),
          duration: this.props.duration,
          easing: this.props.easing && Easing.linear,
        }),
      ]).start(() => this.setState({ toggle: !this.state.toggle }));
    },
    300,
    { trailing: false, leading: true }
  );

  render() {
    return (
      <View style={styles.innerContainer}>
        {isNil(this.props.renderTitle) ? (
          <TouchableOpacity
            style={[styles.sectionView, this.props.titleStyle]}
            onPress={() => {
              this.s
              this.toggle();
            }}
          >
            <Text style={[textStyle.h1, styles.sectionText, this.props.titleTextStyle]}>
              {this.props.title}
            </Text>
          </TouchableOpacity>
        ) : (
          this.props.renderTitle
        )}
        <Animated.View style={{ height: this.state.rendered ? this.state.toggleHeight : null }}>
          <AnimatedFlatList
            style={{
              paddingBottom: this.state.toggleMargin,
              opacity: this.state.rendered || this.state.toggle ? 1 : 0,
            }}
            ListEmptyComponent={this.props.ListEmptyComponent}
            ListFooterComponent={this.props.ListFooterComponent}
            ListHeaderComponent={this.props.ListHeaderComponent}
            data={this.props.data}
            onLayout={this.onListLayout}
            keyExtractor={this.props.keyExtractor}
            renderItem={this.props.renderItem}
          />
        </Animated.View>
      </View>
    );
  }
}

const styles = StyleSheet.create({
  innerContainer: {
    flex: 1,
  },
  sectionView: {
    flexDirection: 'row',
    height: scaleH(50),
    width: screenWidth,
    alignItems: 'center',
  },
  sectionText: {
    marginLeft: scaleW(10),
    textAlign: 'left',
    width: screenWidth - scaleW(60),
  },
  iconMargin: {
    marginRight: scaleW(10),
  },
});
